%{
/* YOODA : Yet another Object Oriented Database
   Copyright (C) 1994  Eric Abecassis

This library is free software; you can redistribute it and/or
modify it under the terms of the GNU Library General Public
License as published by the Free Software Foundation; either
version 2 of the License, or (at your option) any later version.

This library is distributed in the hope that it will be useful,
but WITHOUT ANY WARRANTY; without even the implied warranty of
MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
Library General Public License for more details.

You should have received a copy of the GNU Library General Public
License along with this library; if not, write to the Free
Software Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA. */

#include <stdio.h>
#include <stream.h>
#include <string.h>
#include <fcntl.h>
#include <unistd.h>
#include <signal.h>

#include "simple.hxx"

int line = 1 ;
  
void launchServer(char* s[]) ;
int connect(char* name) ;
int disconnect() ;
Ref< ListX<TstObj> > create(int first, int num, ListX<Cluster>& l_clu) ;
int modify(const Ref< ListX<TstObj> >& l_obj) ;
int destroy(const Ref< ListX<TstObj> >& l_obj) ;
int verify(const Ref< ListX<TstObj> >& l_obj) ;
int verify_destroy(const Ref< ListX<TstObj> >& l_obj) ;
int interrupt_commit() ;
int interrupt_abort() ;


pid_t DB_SERV[4] ;  
extern FILE *yyin ;

extern "C" {
  int yywrap() ;
  int yyparse() ;
}

#if defined(__alpha__) || (defined(__sun__) && !defined(__svr4__))
int yylex() ;
#endif

#if defined(__alpha__) || (defined(__sun__) && !defined(__svr4__))
int yyoutput(int c) ; 
#else
extern "C" { int yyoutput(int c) ; }
#endif

%}

%union  {
	char text[256];
	int  value ;
        }

%token <text> CREATE DELETE MODIFY VERIFY VERIFY_DELETE VERIFY_ALL
%token <text> COMMIT ABORT INTERRUPT_COMMIT INTERRUPT_ABORT 
%token <text> CONNECT DISCONNECT LAUNCH_SERVERS STOP_SERVERS SLEEP IDENT
%token <value> INT

%start cmdlst

%%
cmdlst:
	/* empty */
	| cmdlst cmd
	;

cmd:	create
        | delete
	| modify 
	| verify
	| verify_delete
	| verify_all
        | commit  
        | abort
	| interrupt_commit
	| interrupt_abort
	| connect
	| disconnect
	| launch_servers
	| stop_servers
	| sleep
	;

create: CREATE INT INT
{
  for (int i = 0; i < $2; i++) {
    Ref< ListX<TstObj> > newl = create(Main->size, $3, Main->clusters) ;
    Main->size += newl->count() ;
    Main->list += newl ;
  }
} ;

delete: DELETE
{
  destroy(Main->list[Main->list.count()-1]) ;
} ;

modify: MODIFY
{
  modify(Main->list[Main->list.count()-1]) ;
} ;

verify: VERIFY
{
  if (verify(Main->list[Main->list.count()-1]) == 0) {
    cerr << "verify error : line " << line << endl ;
  }
} ;

verify_delete: VERIFY_DELETE
{
  if (verify_destroy(Main->list[Main->list.count()-1])) {
    cerr << "verify_delete error : line " << line << endl ;
  }

} ;

verify_all: VERIFY_ALL
{
  for (int i = 0; i < Main->list.count(); i++) {
    if (verify(Main->list[i]) == 0) {
      cerr << "verify_all error : line " << line << endl ;
    }
  }
} ;

commit: COMMIT
{
  yoo_commit() ;
}

abort: ABORT
{
  yoo_abort() ;
}

interrupt_commit: INTERRUPT_COMMIT
{
  interrupt_commit() ;
  exit(0) ;
}

interrupt_abort: INTERRUPT_ABORT
{
  interrupt_abort() ;
  exit(0) ;
}

connect: CONNECT IDENT
{
  connect($2) ;
}

disconnect: DISCONNECT
{
  disconnect() ;
}

launch_servers: LAUNCH_SERVERS IDENT 
{
  if ((DB_SERV[0] = fork()) == 0) {
    execlp("yoo_server", "yoo_server", "-b", DB_NAME, "-s", $2, NULL) ;
  }
  DB_SERV[1] = -1 ;
  DB_SERV[2] = -1 ;
  DB_SERV[3] = -1 ;
}
               | LAUNCH_SERVERS IDENT IDENT
{
  if ((DB_SERV[0] = fork()) == 0) {
    execlp("yoo_server", "yoo_server", "-b", DB_NAME, "-s", $2, NULL) ;
  }
  if ((DB_SERV[1] = fork()) == 0) {
    execlp("yoo_server", "yoo_server", "-b", DB_NAME, "-s", $3, NULL) ;
  }
  DB_SERV[2] = -1 ;
  DB_SERV[3] = -1 ;
}
               | LAUNCH_SERVERS IDENT IDENT IDENT
{
  if ((DB_SERV[0] = fork()) == 0) {
    execlp("yoo_server", "yoo_server", "-b", DB_NAME, "-s", $2, NULL) ;
  }
  if ((DB_SERV[1] = fork()) == 0) {
    execlp("yoo_server", "yoo_server", "-b", DB_NAME, "-s", $3, NULL) ;
  }
  if ((DB_SERV[2] = fork()) == 0) {
    execlp("yoo_server", "yoo_server", "-b", DB_NAME, "-s", $4, NULL) ;
  }
  DB_SERV[3] = -1 ;
}
               | LAUNCH_SERVERS IDENT IDENT IDENT IDENT
{
  if ((DB_SERV[0] = fork()) == 0) {
    execlp("yoo_server", "yoo_server", "-b", DB_NAME, "-s", $2, NULL) ;
  }
  if ((DB_SERV[1] = fork()) == 0) {
    execlp("yoo_server", "yoo_server", "-b", DB_NAME, "-s", $3, NULL) ;
  }
  if ((DB_SERV[2] = fork()) == 0) {
    execlp("yoo_server", "yoo_server", "-b", DB_NAME, "-s", $4, NULL) ;
  }
  if ((DB_SERV[3] = fork()) == 0) {
    execlp("yoo_server", "yoo_server", "-b", DB_NAME, "-s", $5, NULL) ;
  }
} ;


stop_servers: STOP_SERVERS
{
  if (DB_SERV[0] != -1) kill(DB_SERV[0], SIGINT) ;
  if (DB_SERV[1] != -1) kill(DB_SERV[1], SIGINT) ;
  if (DB_SERV[2] != -1) kill(DB_SERV[2], SIGINT) ;
  if (DB_SERV[3] != -1) kill(DB_SERV[3], SIGINT) ;
}


sleep: SLEEP INT
{
  sleep($2) ;
}

%%

extern "C" {
  int yyerror(char* s) {
    cerr << s << endl ;
  }
} ;

// Handler pose pour le signal SIGSEGV
void handler(int sig, siginfo_t* ctx, ucontext_t* addr)
{
  VirtualDBMemory::segvhandler(sig, ctx, addr) ;

//   printf("Numero de signal : %d\n", sig) ;
//   printf("siginfo_t : %x\n", ctx) ;
//   printf("Adresse documentee (si_fault) : %x\n", ctx->si_addr) ;
//   printf("Adresse _data._pad[18] : %x\n", ctx->_data._pad[18]) ;
//   printf("Adresse _data._prof._mstate[6] : %x\n", ctx->_data._prof._mstate[6]);
//   exit(1) ;
}

void main() {
  
  yyparse() ;
}

